/*-----------------------------------------------------------------------
		
	Inline Math library : Bounding box implementation (header)
	
	* See imath.h for more information

	By Matt Barnett
		(see References in imath.h)
	
------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------
		
	ibox3 class

	------------
	Description
	------------
		
	Bounding box class based for imath library

	This class performs intersection testing, not "collision detection".
	
	(collisions and intersections are similar, and you need intersections
	 to implement collisions)
	
	-------
	General
	-------

		Everything is public in this class (like the other imath components)
		This class has some non inlined functions

	Orientation
	-----------

		To orient the bbox simply fiddle with
		the quaternion

		eg:		iquaternion4 somerotation;

				//create somerotation with a suitable rotation

				object.GetQuaternion().multiply(somerotation);



---------------------------------------------*/

#ifndef _IBOX3_HEADER_
#define _IBOX3_HEADER_

//#define _IBOX3_TESTWITH_TEAPOT

#ifdef _IBOX3_TESTWITH_TEAPOT
	#include <gl/glut.h>
#else
	#include <gl/gl.h>
	#include <gl/glu.h>
#endif


class ibox3
{
	public:
		
		//assigning
		
		const ibox3 &operator=(const ibox3 &rhs)
		{
			center = rhs.center;
			extents = rhs.extents;	
			pos = rhs.pos;
			quat = rhs.quat;
			bas = rhs.bas;

			return *this;
		}
		
		//------- construction etc ---
		//----------------------------
		
		//constructor
		ibox3() 
		{
			center = zero;
			extents.set(0.01f,0.01f,0.01f);
		}

		//default constructor				
		ibox3(ivector3 &cextents,ivector3 &rotationcenter) 
		{
			center = rotationcenter;
			extents = cextents;
		}

		//copy constructor
		ibox3(const ibox3 &copy)
		{
			center = copy.center;
			extents = copy.extents;
			pos = copy.pos;
			quat = copy.quat;
			bas = copy.bas;
		}

		//destructor
		~ibox3()
		{

		}

		//----------------------------
		//----------------------------

		//------ public attributes ---
		//----------------------------
	
		//you can only change pos/angles directly
		//position, or center of box

		ivector3		pos;		//position of box
		imatrix16		rm;			//rotation matrix (made from quaternion as nessesary)
		iquaternion4	quat;		//

		//dont fuck with the basis outside this class (copy)
		inline ibasis9 getBasis()
		{
			return bas;
		}
		//ditto for extents(copy)
		inline ivector3 getExtents()
		{
			return extents;
		}
		//ditto for center(copy)
		inline ivector3 getCenter()
		{
			return center;
		}

		//----------------------------
		//----------------------------

		const bool intersect(const ibox3 &b) const;	
		
		//===========================================
		//inline bool intersectSphere(const ibox3 &b) const
		//
		//	true for bounding sphere collision detection
		//	between this and b.  optimised to no sqrts.
		//
		//
		//	when inlined this give as big speedup
		//  when we need to call it alot of times
		//  (its an quick exit from collision)
		//============================================*/
		//const bool intersectSphere(const ibox3 &b) const;

		inline bool intersectSphere(const ibox3 &b) const
		{

			//float aradsqr =					killed tempories
			//float bradsqr = 
			//float radsqr = aradsqr + bradsqr;

			//float arad = extents.length();
			//float brad = b.extents.length();
			//float rad = arad + brad;

			ivector3 areal = pos;
			ivector3 breal = b.pos;

			ivector3 temp;

			//because bas can be anything
			//there doesnt seem to be a
			//good way to simplfy here

			temp = bas.x;temp.multiply(center.x);areal.add(temp);
			temp = bas.y;temp.multiply(center.y);areal.add(temp);
			temp = bas.z;temp.multiply(center.z);areal.add(temp);

			temp = b.bas.x;temp.multiply(b.center.x);breal.add(temp);
			temp = b.bas.y;temp.multiply(b.center.y);breal.add(temp);
			temp = b.bas.z;temp.multiply(b.center.z);breal.add(temp);

			temp = breal - areal;

			//as with most of my maths, i dont know why we must *2
			//here. its because the two lengthsqr's on the one side are
			//too small
			if ( temp.lengthsqr() < (extents.lengthsqr() + b.extents.lengthsqr() ) *2 ) 
				return true;
			else
				return false;
			

		}

		const void gldraw();			//draws the oobb (bounding box) (shows what will/should be intersected with)
		const void gldrawSphere();		//draws the bounding sphere

		const void update()
		{
			rm.makefromQuaternion(quat);
			bas = rm.getBasis();
		}

		protected:
		
			ibasis9			bas;		//ortho basis
			ivector3		extents;	//the extents of the box are constant
			ivector3		center;		//center offset
};

#endif