/*-----------------------------------------------------------------------
		
	Inline Math library : Non member function implementation
	
	* See imath.h for more information

	By Matt Barnett
		(see References in imath.h)
	
------------------------------------------------------------------------*/

//-------- headers etc -------
//---------------------------- 

#ifndef _IMATH_CORE_IMP_HEADER_354766003
#define _IMATH_CORE_IMP_HEADER_354766003

//--- non member functions ---
//---------------------------- 
// 
// some may seem silly stuff like
//
// inline float im_sin(float angle) { return sinf(angle) }
// inline float im_sqrt(float val)  { return sqrtf(val)  }
//
// this are interfaces so that these functions can be replaced
// with more efficient versions. (trig functions are replaced with
// lookup tables)
//

//theroy (degrees & radians)
//
//to avoid confusion it is best _NOT_ to convert angle measures.
//Sick with radians untill conversion to degrees is ABSOLUTELY nessesary
//(try not to store degrees, throw them away if appropriate)

//converts radians to degrees
inline float im_r2d(float angle)
{
	return 	(angle * _180_over_pi);
}

//converts degrees to radians
inline float im_d2r(float angle)
{
	return	(angle * _pi_over_180);
}

//random number generation
//seed with the generator (custom seed
inline void im_random_seed(unsigned int seed)
{
	srand( seed );
}
//seed with time
inline void im_random_seed_time()
{
	im_random_seed( GetTickCount() );
}

//random number retrival functions
//returns a float between 0 & 1

inline float im_random_num()
{
	return ((float)rand()/(RAND_MAX+1));
}

//all based on im_random_num
inline float im_random_num(float range)   { return im_random_num() * range;}		//returns a float between 0 & range
inline float im_random_clamp()			  { return -1 + im_random_num() * 2;  }		//returns a float in the range -1.0f -> + 1.0f
inline float im_random_clamp(float range) { return im_random_clamp() * range; }		//returns a float between -range & +range

//sqrt
inline float im_sqrt(float in)
{
	return sqrtf(in);
}

inline float im_sin(float in)
{
	//return sinf(in);
	return sintable.lookup(in);
}

inline float im_cos(float in)
{
	//return cosf(in);
	return costable.lookup(in);
}

inline float im_acos(float in)
{
	return acosf(in);
}

inline float im_tan(float in)
{
	//return tanf(in);
	return tantable.lookup(in);
}

//---------------------------- 
//----------------------------

#endif