/*---------------------------------------------
		
	MatNN library 

	By Matt Barnett
		(see References)

  
	-----------
	usage
	-----------

		I find that this is the section that i most often
		look for in other peoples docs, and neglect in my own.

		Look at the examples for usage comments, or examin
		the public interface.

	-----------
	description
	-----------

		This is not really a library yet, but its designed so it
		can be extended (hence the namespace) , and is as general as possible.
	
	  
		The Neural Net Specification
		(this is the same as the this library's
		pre-cursor, my other nn classes)

		* Neurons are made in layers

			input  layer  (any number of neurons)
			hidden layers (each has the same number of neurons)
			output layer  (any number of neurons)

		* Each layer (starting with the input layer) uses its neurons
		  to calculate the outputs for all the neurons in this layer.
		  This is stored in an array which the subsequent layer "knows".

		* The inital layer has no neuon inputs, this is where the input
		  from the "environment" enters the net. The input layer
		  has a pointer to inputs that are specified by the client

	-----------
	portability
	-----------

		* Depends on imath.h (see imath.h) (needs OpenGL, this is bad for a class a general as this)
		* Error messages need windows, hack the error class to support non windows
		* Neural nets are written to text files for easy editing and portability

	------------
	class goals
	------------

		NeuralNet : Serves as a base class for more specific neural nets (basic neural net function)
					Will not contain any code for training/adjustment of the net.

	------------
	class design
	------------

		The design is such that, the net class constructs itself
		appropriatly for the given parameters and is then ready to
		run.  This helps keep the rest of the interface small.

		Neural nets are written to a formatted text file for storage.
		This makes it easy to understand / edit and should make it
		more portable. (ie no struct size depenence)

		* No consideration given to const/non const neural nets/neurons/layers	( YET ! )
		* There is no default constructor.

		* Class design leaves some bounds checking to the client !!!!!!!
		  
		  (s/he simply has to stay within the bounds of his own arrays and be able
		   to remeber/work out the size of the array, (only once) )


	-----------
	References:	
	-----------

		* See imath.h, myerror.h
		
	
	
---------------------------------------------*/



//-------- headers etc -------
//---------------------------- 

#ifndef MATNN_MAIN_H_HEADER_012023254
#define MATNN_MAIN_H_HEADER_012023254

#include "../leakdetect.h"

#include <iostream>

//using std::cout;
//using std::endl;

#include <math.h>						//fpu maths
#include <cassert>						//assert
//#include <windows.h>					//opengl needs windows (comment for non WIN32 platforms)
//#include <gl/gl.h>					

#include "matnn_neuron.h"
#include "../imath/imath.h"
#include "../myerror.h"

//----------------------------
//----------------------------


//these limits are quite arbitrary
//just to prevent the user making an insane
//net just to test the memory handling (which isnt that hot)

#define MATNN_MAX_HIDDEN_LAYERS		256				//256 seems a reasonable limit for these
#define MATNN_MAX_IN_NEURONS		256				//
#define MATNN_MAX_OUT_NEURONS		256				//
#define MATNN_MAX_HIDDEN_NEURONS	256				//


namespace MatLayeredNeuralNetLib
{
	class NeuralNet
	{
		//neural net public interface
		
		public:
			
			//the client is responsible for storage of input & output arrays
			//but they only need to be set once, and then the you change the input
			//just be fiddling the array.

			NeuralNet(NeuralNet &copy);				//copy
			NeuralNet &operator=(NeuralNet &rhs);	//assign

			NeuralNet(int in_neurons,int out_neurons,int hidden_neurons,int hidden_layers,float *in_finput,float *in_foutput);
			~NeuralNet();

			void RunNN();												//runs the neural net
			void SetResponse(float in_sig_response);					//alters the response constant for all neurons and layers in the net
			void SetInputOutput(float *in_finput,float *in_foutput);	//alters input/output arrays (null checking)
			void Randomize();											//dangerous to call on a trained net (it totally randomises weights to -1.0 + 1.0)



		protected:
		
			int num_input_neurons;					
			int num_output_neurons;					
			int num_hidden_neurons;					
			int num_hidden_layers;					

			NeuronLayer	*input_layer;				//will point to the first layer
			NeuronLayer	*output_layer;				//will point to the last layer
			NeuronLayer	**parray_hidden_layers;		//an array of pointers for the hidden layers
		
		private:
			float *input;
			float *output;

			float sig_response;
	};
	
	
	class TrainableNeuralNet : public NeuralNet
	{
		public:

			TrainableNeuralNet(int in_neurons,int out_neurons,int hidden_neurons,int hidden_layers,float *in_finput,float *in_foutput) : NeuralNet(in_neurons,out_neurons,hidden_neurons,hidden_layers,in_finput,in_foutput) {}
			~TrainableNeuralNet() {}
		
			int Mutate(float mutation_chance,float mutation_extent);		//returns the number of mutations that occured
			//ref or copy?
			//TrainableNeuralNet Clone()
			//TrainableNeuralNet Cross(TrainableNeuralNet &mate)
			//void Merge()
	};
	
}

#endif